;****************************************************************************             
;*** NIXIE.ASM   4 x B7971 Tubes
;*** 
;*** Author: (C)-2002 Christian Mohr // Christian@Mohr.se
;*** 
;*** May not be published in any form without permission from the author!
;****************************************************************************            
;*** CPU: 16C84
;*** CLOCK: 10MhZ Xtal
;****************************************************************************            


; MODE[0]=1 TEST/FAST-MODE ON
;
; MAIN LOOP'S
; 0x00 = DISPLAY TIME
; 0x01 = MAIN MENU - SET
; 0x02 = MAIN MENU - TEST
; 0x10 = SET 10 x Hour
; 0x11 = SET  1 x Hour
; 0x12 = SET 10 x Minute
; 0x13 = SET  1 x Minute



        Radix   DEC
        EXPAND

        include "p16f84.inc"


        __CONFIG 01h



;****************************************************************************             
;*** PORT DECLARATIONS
;****************************************************************************            

#define NCK	PORTA,0		; NIXIE CLOCK
#define NEN	PORTA,1		; NIXIE ENABLE
#define NDA	PORTA,2		; NIXIE DATA
#define NST	PORTA,3		; NIXIE STROBE

#define I2C_C	PORTB,0		; I2C Clock
#define I2C_D	PORTB,1		; I2C Data
;#define DS1	PORTB,2		; D1
;#define DS2	PORTB,3		; D2
#define B_MNU	PORTB,5		; BUTTON 0 = MENU/OK
#define B_PL	PORTB,6		; BUTTON 1 = +
#define B_MI	PORTB,7		; BUTTON 2 = -

;****************************************************************************             
;*** RAM DECLARATIONS
;****************************************************************************            



MODE		set	0x1D	; Mode Flag
SCROFF		set	0x1E	; Scrolling Hello Offset
CURSOR		set	0x1F    ; CURSOR[3..0] = Illuminates the cursors!
IOBYTE  	set     0x20	; I/O DATA STORAGE
S1		set	0x21	; SECONDS 1
S0		set	0x22	; SECONDS 10
M1		set	0x23	; MINUTES 1
M0		set	0x24	; MINUTES 10
H1		set	0x25	; HOURS 1
H0		set	0x26	; HOURS 10
C3		set	0x27	; CHAR 3	; DO NOT MOVE!
C2		set	0x28	; CHAR 2	; DO NOT MOVE!
C1		set	0x29	; CHAR 1	; DO NOT MOVE!
C0		set	0x2A	; CHAR 0	; DO NOT MOVE!
_N		set	0x2B	; BITCOUNTER
T3		set	0x2C	; TEMP. STORAGE
T2		set	0x2D	; TEMP. STORAGE
T1		set	0x2E	; TEMP. STORAGE
T0		set	0x2F	; TEMP. STORAGE


;****************************************************************************             
;*** Vectors
;****************************************************************************            

        ORG     0x00
	goto	Start
        ORG     0x04
	goto	Interrupt


;****************************************************************************             
;*** STRINGS
;****************************************************************************            

HELLOSTR:
	ADDWF PCL, F
	DT "    PLEASE ENJOY THIS NIXIE-CLOCK BY C MOHR    ", 00H, 00H, 00H, 00H, 00H


;****************************************************************************             
;*** BIN -> NIXIE
;****************************************************************************            

NR0:    addwf   PCL,F
        retlw   b'10001000'     ; 0   
        retlw   b'00000000'     ; 1
        retlw   b'01000100'     ; 2
        retlw   b'11000000'     ; 3
        retlw   b'01000100'     ; 4   
        retlw   b'00100100'     ; 5
        retlw   b'01000100'     ; 6
        retlw   b'10001000'     ; 7
        retlw   b'01000100'     ; 8   
        retlw   b'01000100'     ; 9


NR1:    addwf   PCL,F
        retlw   b'11111100'     ; 0   
        retlw   b'01100000'     ; 1
        retlw   b'11011000'     ; 2
        retlw   b'10110000'     ; 3
        retlw   b'01100100'     ; 4   
        retlw   b'10010100'     ; 5
        retlw   b'10111100'     ; 6
        retlw   b'10000000'     ; 7
        retlw   b'11111100'     ; 8   
        retlw   b'11110100'     ; 9


CHR0:   addwf   PCL,F
        retlw   b'01000100'     ; A 
        retlw   b'01010000'     ; B 
        retlw   b'00000000'     ; C
        retlw   b'00010000'     ; D
        retlw   b'00000100'     ; E
        retlw   b'00000100'     ; F
        retlw   b'01000000'     ; G
	retlw   b'01000100'     ; H
        retlw   b'00010000'     ; I
        retlw   b'00000000'     ; J
        retlw   b'10100100'     ; K
        retlw   b'00000000'     ; L
        retlw   b'10000000'     ; M
        retlw   b'00100000'     ; N
        retlw   b'00000000'     ; O
        retlw   b'01000100'     ; P
        retlw   b'00100000'     ; Q
        retlw   b'01100100'     ; R
        retlw   b'01000100'     ; S
        retlw   b'00010000'     ; T
        retlw   b'00000000'     ; U
        retlw   b'10001000'     ; V
        retlw   b'00101000'     ; W
        retlw   b'10101000'     ; X
        retlw   b'10001000'     ; Y
        retlw   b'10001000'     ; Z

CHR1:	addwf   PCL,F
        retlw   b'11101100'     ; A 
        retlw   b'11110001'     ; B 
        retlw   b'10011100'     ; C
        retlw   b'11110001'     ; D
        retlw   b'10011100'     ; E
        retlw   b'10001100'     ; F
        retlw   b'10111100'     ; G
        retlw   b'01101100'     ; H
        retlw   b'10010001'     ; I
        retlw   b'01111000'     ; J
        retlw   b'00001100'     ; K
        retlw   b'00011100'     ; L
        retlw   b'01101110'     ; M
        retlw   b'01101110'     ; N
        retlw   b'11111100'     ; O
        retlw   b'11001100'     ; P
        retlw   b'11111100'     ; Q
        retlw   b'11001100'     ; R
        retlw   b'10110100'     ; S
        retlw   b'10000001'     ; T
        retlw   b'01101100'     ; U
        retlw   b'00001100'     ; V
        retlw   b'01101100'     ; W
        retlw   b'00000010'     ; X
        retlw   b'00000010'     ; Y
        retlw   b'10010000'     ; Z


; ******************************************************************
; *** DISPLAY DATA ON NIXIETUBES
; ******************************************************************
DNIXIE:	BSF	NEN

	MOVLW	.4
	MOVWF	T2

	MOVLW	0x27
	MOVWF	FSR

DNIX1:	MOVLW	.32		; <SPC> ASC32    ---> T0,T1 = 0
	SUBWF	INDF,W
	BTFSS	STATUS,Z
	GOTO	DNLA
	CLRF	T0
	CLRF	T1
	GOTO	DNLX

DNLA:	MOVLW	.45		; - ASC45    ---> T0,T1 = ?,?
	SUBWF	INDF,W
	BTFSS	STATUS,Z
	GOTO	DNLB
	MOVLW	b'01000100'
	MOVWF	T0
	CLRF	T1
	GOTO	DNLX

DNLB:	MOVLW	.64		; A-Z            ---> CHR0 & CHR1
	SUBWF	INDF,W
	BTFSS	STATUS,C
	GOTO	DNLC
	MOVLW	.65
	SUBWF	INDF,W
	CALL	CHR0
	MOVWF	T0
	MOVLW	.65
	SUBWF	INDF,W
	CALL	CHR1
	MOVWF	T1
	GOTO	DNLX

DNLC:	MOVF	INDF,W  	; 0-9 = ASC48-57 ---> NR0 & NR1
	CALL	NR0
	MOVWF	T0
	MOVF	INDF,W
	CALL	NR1
	MOVWF	T1


DNLX:	RRF	CURSOR,F
	BTFSC	STATUS,C	; CURSOR
	BSF	T0,1

	MOVLW	.8
	MOVWF	_N
DNL0:	RRF	T0,F
	BCF	NDA
	BTFSC	STATUS,C
	BSF	NDA
	NOP
	BSF	NCK
	NOP
	BCF	NCK
	DECFSZ	_N,F
	GOTO	DNL0

	MOVLW	.8
	MOVWF	_N
DNL1:	RRF	T1,F
	BCF	NDA
	BTFSC	STATUS,C
	BSF	NDA
	NOP
	BSF	NCK
	NOP
	BCF	NCK
	DECFSZ	_N,F
	GOTO	DNL1


	INCF	FSR,F		; MINSKA PEKAREN C3->C2->C1->C0
	DECFSZ	T2,F		; KR 4 VARV
	GOTO	DNIX1

	BSF	NST		; OK, SKICKA EN STROBE TILL DRIVKORTEN
	NOP
	NOP
	BCF	NST

	RETURN

; ******************************************************************
; *** SAY HELLO!
; ******************************************************************

HELLO:	CLRF	SCROFF

HELLO1:	MOVF	SCROFF,W
	CALL	HELLOSTR
	MOVWF	C0

	MOVF	SCROFF,W
	ADDLW	.1
	CALL	HELLOSTR
	MOVWF	C1

	MOVF	SCROFF,W
	ADDLW	.2
	CALL	HELLOSTR
	MOVWF	C2

	MOVF	SCROFF,W
	ADDLW	.3
	CALL	HELLOSTR
	MOVWF	C3

	MOVF	C3,F		; END OF MESSAGE?
	BTFSC	STATUS,Z
	GOTO	HELLOX

	CLRF	CURSOR
	CALL	DNIXIE
	CALL	D100ms
	CALL	D100ms
	CALL	D100ms

	INCF	SCROFF,F

	GOTO	HELLO1

HELLOX:	RETURN



; ******************************************************************
; *** SET PORTS TO INPUT / OUTPUT AND INITIALIZE VARIABLES
; ******************************************************************
INIT:	bsf     STATUS,RP0     ; Select Page 1 for Tris access

	bcf	NEN
	bcf	NDA
	bcf	NST
	bcf	NCK
	bcf	I2C_C
	bsf	B_MNU
	bsf	B_PL
	bsf	B_MI
	bcf	OPTION_REG,7
        bcf     STATUS,RP0     ; select Page 0 for Port Access

	bcf	NEN
	bcf	NCK
	bcf	NST
	bsf	B_MNU
	bsf	B_PL
	bsf	B_MI

        RETURN

; ******************************************************************
; *** ISR
; ******************************************************************

Interrupt:

	RETFIE


; ******************************************************************
; *** READ TIME FROM RTC
; ******************************************************************
RTIME:  CALL    I2CSTA
        MOVLW   0xA0    ; address
        CALL    I2CWB
        MOVLW   0x01    ; 01 = HS/SS/MM/HH
        CALL    I2CWB
        CALL    I2CSTO

        CALL    I2CSTA
        MOVLW   0xA1
        CALL    I2CWB
	BTFSS	MODE,0	; SPEEDUP IN TESTMODE!
        CALL    I2CRB
;        MOVWF   HS	; DISCARD 100's SECONDS
        CALL    I2CRB
        MOVWF   S0
        CALL    I2CRB
        MOVWF   M0
        CALL    I2CRL   ; READ LAST = NO I2CSTO NEEDED
        MOVWF   H0

        ; TAKE CARE OF TIME DATA
        MOVF    S0,W
        MOVWF   S1
        MOVLW   0x0F
        ANDWF   S0,F
        RRF     S1,F
        RRF     S1,F
        RRF     S1,F
        RRF     S1,F
        MOVLW   0x0F
        ANDWF   S1,F

        MOVF    M0,W
        MOVWF   M1
        MOVLW   0x0F
        ANDWF   M0,F
        RRF     M1,F
        RRF     M1,F
        RRF     M1,F
        RRF     M1,F
        MOVLW   0x0F
        ANDWF   M1,F

        MOVF    H0,W
        MOVWF   H1
        MOVLW   0x0F
        ANDWF   H0,F
        RRF     H1,F
        RRF     H1,F
        RRF     H1,F
        RRF     H1,F
        MOVLW   0x0F
        ANDWF   H1,F
	RETURN




; ******************************************************************
; *** MAIN PROGRAM BEGINS HERE!!!
; ******************************************************************
Start:	; MAIN PROGRAM INITIALIZATION POINT

        CALL    INIT
	CALL	I2CINIT

        CALL    I2CSTO

	BCF	MODE,0

        call    D500ms

        CALL    I2CSTA          ; INITIALIZE 8583 CONFIG WORD
        MOVLW   0xA0
        CALL    I2CWB
        MOVLW   0x00
        CALL    I2CWB
        MOVLW   0x00
        CALL    I2CWB
        CALL    I2CSTO

	CALL	HELLO

	CALL	D500ms
	CALL	D500ms


; *** NORMAL TIME DISPLAY
M_00:	CALL	D100ms
	CLRF	CURSOR	
	CALL	RTIME	; Read Time From RTC
	MOVF	H1,W
	MOVWF	C0
	MOVF	H0,W
	MOVWF	C1
	MOVF	M1,W
	MOVWF	C2
	MOVF	M0,W
	MOVWF	C3
	CALL	DNIXIE

	BTFSC	B_MNU	; CHECK FOR MENU-BUTTON
	GOTO	M_00
	CALL	D500us
	BTFSS	B_MNU	; DEBOUNCE
	GOTO	$-1
	GOTO	M_01


; *** SET MENU ---> [+] = ENTER SET MODE
M_01:	CALL	D100ms
	CLRF	CURSOR
	MOVLW	'S'	; Display "SET "
	MOVWF	C0
	MOVLW	'E'
	MOVWF	C1
	MOVLW	'T'
	MOVWF	C2
	MOVLW	' '
	MOVWF	C3
	CALL	DNIXIE

	BTFSC	B_MNU	; MENU?
	GOTO	M_01_0
	BTFSS	B_MNU	; DEBOUNCE
	GOTO	$-1
	GOTO	M_02	; ELLER TILL NSTA MENY HR!!!

M_01_0:	BTFSC	B_PL	; PLUS?
	GOTO	M_01
	BTFSS	B_PL	; DEBOUNCE
	GOTO	$-1
	GOTO	M_10


; *** TEST MENU --->  [+] = ENTER TESTMODE
M_02:	CALL	D100ms
	CLRF	CURSOR
	MOVLW	'T'	; Display "TEST"
	MOVWF	C0
	MOVLW	'E'
	MOVWF	C1
	MOVLW	'S'
	MOVWF	C2
	MOVLW	'T'
	MOVWF	C3
	CALL	DNIXIE

	BTFSC	B_MNU	; MENU?
	GOTO	M_02_0
	BTFSS	B_MNU	; DEBOUNCE
	GOTO	$-1
	GOTO	M_00	; ELLER TILL NSTA MENY HR!!!

M_02_0:	BTFSC	B_PL	; PLUS?
	GOTO	M_02
	BTFSS	B_PL	; DEBOUNCE
	GOTO	$-1
	BSF	MODE,0	; SET TESTMODE ON HERE!!!	
	GOTO	M_00




M_10:	CALL	D100ms
	CLRF	CURSOR
	BSF	CURSOR,3
	MOVF	H1,W
	MOVWF	C0
	MOVF	H0,W
	MOVWF	C1
	MOVF	M1,W
	MOVWF	C2
	MOVF	M0,W
	MOVWF	C3
	CALL	DNIXIE
	
	BTFSC	B_MNU	; MENU?
	GOTO	M_10_0
	BTFSS	B_MNU	; DEBOUNCE
	GOTO	$-1
	GOTO	M_11

M_10_0:	BTFSC	B_PL	; PLUS?
	GOTO	M_10_1
	BTFSS	B_PL	; DEBOUNCE
	GOTO	$-1
	; CHECK WRAP 0-1-2->0
	INCF	H1,F
	MOVLW	.3
	SUBWF	H1,W
	BTFSS	STATUS,Z
	GOTO	M_10
	MOVLW	.0
	MOVWF	H1
	GOTO	M_10

M_10_1:	BTFSC	B_MI	; MINUS?
	GOTO	M_10
	BTFSS	B_MI	; DEBOUNCE
	GOTO	$-1
	DECF	H1,F
	; CHECK WRAP 2-1-0->2
	BTFSS	H1,2
	GOTO	M_10
	MOVLW	.2
	MOVWF	H1
	GOTO	M_10
	


M_11:	CALL	D100ms
	CLRF	CURSOR
	BSF	CURSOR,2
	MOVF	H1,W
	MOVWF	C0
	MOVF	H0,W
	MOVWF	C1
	MOVF	M1,W
	MOVWF	C2
	MOVF	M0,W
	MOVWF	C3
	CALL	DNIXIE
	
	BTFSC	B_MNU	; MENU?
	GOTO	M_11_0
	BTFSS	B_MNU	; DEBOUNCE
	GOTO	$-1
	GOTO	M_12

M_11_0:	BTFSC	B_PL	; PLUS?
	GOTO	M_11_1
	BTFSS	B_PL	; DEBOUNCE
	GOTO	$-1
	; CHECK WRAP
; 2DO --->  00-01-...-23-24-00
	INCF	H0,F
	MOVLW	.10
	SUBWF	H0,W
	BTFSS	STATUS,Z
	GOTO	M_11
	MOVLW	.0
	MOVWF	H0
	GOTO	M_11

M_11_1:	BTFSC	B_MI	; MINUS?
	GOTO	M_11
	BTFSS	B_MI	; DEBOUNCE
	GOTO	$-1
	DECF	H0,F
	; CHECK WRAP
; MORE CHECK!!!
	BTFSS	H0,4
	GOTO	M_11
	MOVLW	.9
	MOVWF	H0
	GOTO	M_11



M_12:	CALL	D100ms
	CLRF	CURSOR
	BSF	CURSOR,1
	MOVF	H1,W
	MOVWF	C0
	MOVF	H0,W
	MOVWF	C1
	MOVF	M1,W
	MOVWF	C2
	MOVF	M0,W
	MOVWF	C3
	CALL	DNIXIE
	
	BTFSC	B_MNU	; MENU?
	GOTO	M_12_0
	BTFSS	B_MNU	; DEBOUNCE
	GOTO	$-1
	GOTO	M_13

M_12_0:	BTFSC	B_PL	; PLUS?
	GOTO	M_12_1
	BTFSS	B_PL	; DEBOUNCE
	GOTO	$-1
	; CHECK WRAP
	INCF	M1,F
	MOVLW	.6
	SUBWF	M1,W
	BTFSS	STATUS,Z
	GOTO	M_12
	MOVLW	.0
	MOVWF	M1
	GOTO	M_12

M_12_1:	BTFSC	B_MI	; MINUS?
	GOTO	M_12
	BTFSS	B_MI	; DEBOUNCE
	GOTO	$-1
	DECF	M1,F
	; CHECK WRAP
	BTFSS	M1,3
	GOTO	M_12
	MOVLW	.5
	MOVWF	M1
	GOTO	M_12



M_13:	CALL	D100ms
	CLRF	CURSOR
	BSF	CURSOR,0
	MOVF	H1,W
	MOVWF	C0
	MOVF	H0,W
	MOVWF	C1
	MOVF	M1,W
	MOVWF	C2
	MOVF	M0,W
	MOVWF	C3
	CALL	DNIXIE
	
	BTFSC	B_MNU	; MENU?
	GOTO	M_13_0
	BTFSS	B_MNU	; DEBOUNCE
	GOTO	$-1
	GOTO	TSAVE	; SET DONE!

M_13_0:	BTFSC	B_PL	; PLUS?
	GOTO	M_13_1
	BTFSS	B_PL	; DEBOUNCE
	GOTO	$-1
	; CHECK WRAP
	INCF	M0,F
	MOVLW	.10
	SUBWF	M0,W
	BTFSS	STATUS,Z
	GOTO	M_13
	MOVLW	.0
	MOVWF	M0
	GOTO	M_13

M_13_1:	BTFSC	B_MI	; MINUS?
	GOTO	M_13
	BTFSS	B_MI	; DEBOUNCE
	GOTO	$-1
	DECF	M0,F
	; CHECK WRAP
	BTFSS	M0,4
	GOTO	M_13
	MOVLW	.9
	MOVWF	M0
	GOTO	M_13



TSAVE	; STORE TIME TO 8583

	MOVLW	'S'
	MOVWF	C0
	MOVLW	'A'
	MOVWF	C1
	MOVLW	'V'
	MOVWF	C2
	MOVLW	'E'
	MOVWF	C3

	CALL	DNIXIE
	CALL	D500ms

        CALL    I2CSTA
        MOVLW   0xA0
        CALL    I2CWB
        MOVLW   0x01
        CALL    I2CWB
        MOVLW   0x00    ; 100/s SECONDS = 00

	BTFSS	MODE,0	; SPEEDUP IN TESTMODE!
	CALL    I2CWB

        MOVLW   0x00    ; SECONDS = 00
        CALL    I2CWB
        MOVF    M1,W    ; MINUTES
        MOVWF   T0
        RLF     T0,F
        RLF     T0,F
        RLF     T0,F
        RLF     T0,F
        MOVLW   0xF0
        ANDWF   T0,F
        MOVLW   0x0F
        ANDWF   M0,W
        IORWF   T0,W
        CALL    I2CWB
        MOVF    H1,W    ; HOURS
        MOVWF   T0
        RLF     T0,F
        RLF     T0,F
        RLF     T0,F
        RLF     T0,F
        MOVLW   0xF0
        ANDWF   T0,F
        MOVLW   0x0F
        ANDWF   H0,W
        IORWF   T0,W
        ANDLW   0x3F
        CALL    I2CWB
        CALL    I2CSTO

	GOTO	M_00	; DONE - RETURN TO TIME DISPLAY


ende	goto	ende







;****************************************************************************             
;*** Modules
;****************************************************************************            


	include "_delay.asm"
	include "_i2c.asm"
	include "_misc.asm"


;
;****************************************************************************
;****************************************************************************
;****************************************************************************

        END
